@@ -64,9 +64,8 @@ class DeviceLogParser(Device):
                     header_name = "{} ({})".format(header, unit[:-1])
                     headers.append(header_name)
                     dtypes[header_name] = dtype
-
                 df = pandas.read_csv(path, sep=";", low_memory=True, skiprows=start_row, nrows=nrows,
-                                     header=0, names=headers, dtype=dtypes)
+                                     header=0, names=headers, dtype=dtypes, error_bad_lines=False)
                 data_frames.append(df)
 
         data_frame = pandas.concat(data_frames, copy=True)
