@@ -23,7 +23,8 @@ from kamzik3.snippets.snippetsControlLoops import control_asyncore_loop, control
 from kamzik3.snippets.snippetsDecorators import expose_method
 from kamzik3.snippets.snippetsTimer import CallbackTimer
 from kamzik3.snippets.snippetsYaml import YamlDumper, YamlConfigSerializer
-from multiprocessing import Process
+from multiprocessing import Process, Event
+
 
 class DeviceSession(Device):
     LOG_DEVICE_OUTPUT_FILE = u"devices.log"
@@ -140,7 +141,7 @@ class DeviceSession(Device):
 
             self.start_polling()
             self.set_status(STATUS_CONFIGURED)
-            self.logger.info(u"Device configuration took {} sec.".format(time.time() - start_at))
+            self.logger.info(f"Device configuration took {time.time() - start_at} sec.")
 
         _finish_configuration()
 
@@ -164,14 +165,16 @@ class DeviceSession(Device):
     def command(self, command, callback=None, with_token=False, returning=True):
         raise DeviceError("Device does not accept any commands.")
 
-    def get_device(self, device_id):
+    def get_device(self, device_id, search_master=True):
         try:
             return self.devices[device_id]
         except KeyError:
-            master_server, master_port = self.config.get("master_server", False), self.config.get("master_port", False)
-            if master_server and master_port:
-                return DeviceClient(master_server, master_port, device_id)
-            raise DeviceUnknownError("Device {} was not found.".format(device_id))
+            if search_master:
+                master_server, master_port = self.config.get("master_server", False), \
+                                             self.config.get("master_port", False)
+                if master_server and master_port:
+                    return DeviceClient(master_server, master_port, device_id)
+            raise DeviceUnknownError(f"Device {device_id} was not found.")
 
     def change_device_id(self, device_id, new_device_id):
         device = self.get_device(device_id)
@@ -186,7 +189,7 @@ class DeviceSession(Device):
             device.session = self
             self.set_value(ATTR_DEVICES_COUNT, len(self.devices))
         else:
-            raise DeviceError("Device {} is already registered within session.".format(device.device_id))
+            raise DeviceError(f"Device {device.device_id} is already registered within session.")
 
     def remove_device(self, device):
         assert isinstance(device, Device)
@@ -226,21 +229,21 @@ class DeviceSession(Device):
         return filtered_devices
 
     def set_log_directory(self, value):
-        self.logger.info(u"Setting log directory to: {}".format(value))
+        self.logger.info(f"Setting log directory to: {value}")
         if not os.path.exists(value):
-            self.logger.info(u"Directory {} does not exists, trying to create it.".format(value))
+            self.logger.info(f"Directory {value} does not exists, trying to create it.")
             os.makedirs(value)
 
     def set_resource_directory(self, value):
-        self.logger.info(u"Setting resource directory to: {}".format(value))
+        self.logger.info(f"Setting resource directory to: {value}")
         if not os.path.exists(value):
-            self.logger.info(u"Directory {} does not exists, trying to create it.".format(value))
+            self.logger.info(f"Directory {value} does not exists, trying to create it.")
             os.makedirs(value)
 
     def set_cache_directory(self, value):
         self.logger.info(u"Setting cache directory to: {}".format(value))
         if not os.path.exists(value):
-            self.logger.info(u"Directory {} does not exists, trying to create it.".format(value))
+            self.logger.info(f"Directory {value} does not exists, trying to create it.")
             os.makedirs(value)
 
     def set_console_log_directory(self, value):
@@ -248,9 +251,9 @@ class DeviceSession(Device):
         if log_directory is None:
             raise DeviceError(u"Cannot set console log directory. Attribute 'Log directory' is not set.")
         log_directory = os.path.join(log_directory, value)
-        self.logger.info(u"Setting console log directory to: {}".format(log_directory))
+        self.logger.info(f"Setting console log directory to: {log_directory}")
         if not os.path.exists(log_directory):
-            self.logger.info(u"Directory {} does not exists, trying to create it.".format(log_directory))
+            self.logger.info(f"Directory {log_directory} does not exists, trying to create it.")
             os.makedirs(log_directory)
 
         handler = get_console_handler()
@@ -269,9 +272,9 @@ class DeviceSession(Device):
             raise DeviceError(u"Cannot set gui log directory. Attribute 'Log directory' is not set.")
         log_directory = os.path.join(log_directory, value)
 
-        self.logger.info(u"Setting gui log directory to: {}".format(log_directory))
+        self.logger.info(f"Setting gui log directory to: {log_directory}")
         if not os.path.exists(log_directory):
-            self.logger.info(u"Directory {} does not exists, trying to create it.".format(log_directory))
+            self.logger.info(f"Directory {log_directory} does not exists, trying to create it.")
             os.makedirs(log_directory)
 
         handler = get_console_handler()
@@ -326,9 +329,9 @@ class DeviceSession(Device):
             raise DeviceError(u"Cannot set gui log directory. Attribute 'Log directory' is not set.")
         log_directory = os.path.join(log_directory, value)
 
-        self.logger.info(u"Setting gui log directory to: {}".format(log_directory))
+        self.logger.info(f"Setting gui log directory to: {log_directory}")
         if not os.path.exists(log_directory):
-            self.logger.info(u"Directory {} does not exists, trying to create it.".format(log_directory))
+            self.logger.info(f"Directory {log_directory} does not exists, trying to create it.")
             os.makedirs(log_directory)
 
         handler = get_console_handler()
@@ -383,7 +386,7 @@ class DeviceSession(Device):
         """
         self.stop_polling()
         if self.get_value(ATTR_ALLOW_ATTRIBUTE_LOG):
-            logger = self.get_device("AttributeLogger")
+            logger = self.get_device("AttributeLogger", search_master=False)
             if logger is not None:
                 logger.stop()
 
@@ -472,9 +475,17 @@ class DeviceSession(Device):
                 elif os.path.isdir(file_path):
                     shutil.rmtree(file_path)
             except Exception as e:
-                self.logger.error('Failed to delete %s. Reason: %s' % (file_path, e))
+                self.logger.error(f"Failed to delete {file_path}: {e}")
 
-def server_process(conf, stop_event):
+def server_process(conf: str, stop_event: Event) -> Process:
+    """
+    For the reason of having multiple Processes handling multiple devices, we can start each server in it's own Process.
+    This function will start Server in new Process and return that Process.
+    If stop event is set, then stop session.
+    :param conf: path to the config file
+    :param stop_event: event
+    :return: Process
+    """
     def _process(conf, stop_event):
         import kamzik3
         import yaml
@@ -486,7 +497,10 @@ def server_process(conf, stop_event):
         kamzik3.session.set_config(conf)
         kamzik3.session.start_control_loops()
         while not stop_event.is_set():
-            sleep(1)
+            try:
+                sleep(1)
+            except KeyboardInterrupt:
+                break
         kamzik3.session.stop()
 
     return Process(target=_process, args=(conf, stop_event))
