@@ -219,7 +219,8 @@ class DeviceSmaractMcs1Channel(DeviceChannel):
         self.create_attribute(ATTR_CALIBRATED, readonly=True, default_type=np.bool)
         self.create_attribute(ATTR_REFERENCED, readonly=True, default_type=np.bool)
         self.create_attribute(ATTR_MOVEMENT_LOCKED, readonly=True, default_type=np.bool)
-        self.create_attribute(ATTR_SENSOR_MODE, readonly=True, default_type=self.sensor_modes.keys())
+        self.create_attribute(ATTR_SENSOR_MODE, default_value=self.device.get_value(ATTR_SENSOR_MODE),
+                              readonly=True, default_type=self.sensor_modes.keys())
         self.create_attribute(ATTR_SENSOR_TYPE, default_type=self.sensor_types, set_function=self.set_sensor_type)
         self.create_attribute(ATTR_SENSOR_PRESENT, readonly=True, default_type=np.bool)
         self.create_attribute(ATTR_ENCODER_BASE_UNIT, readonly=True)
@@ -232,7 +233,7 @@ class DeviceSmaractMcs1Channel(DeviceChannel):
                               max_value=18.5e3, unit="Hz",
                               set_function=self.set_step_frequency)
         self.create_attribute(ATTR_STEP_AMPLITUDE, default_value=1000, default_type=np.uint32, min_value=1,
-                              max_value=65535,
+                              max_value=4095,
                               set_function=self.set_step_amplitude)
         self.create_attribute(ATTR_ACCELERATION, default_type=np.int64, min_value=0, max_value=100e9,
                               set_function=self.set_acceleration)
@@ -242,6 +243,10 @@ class DeviceSmaractMcs1Channel(DeviceChannel):
                               set_function=self.set_safe_direction)
         self.create_attribute(ATTR_HOLDING_TIME, default_value=0, default_type=np.uint16, min_value=0, max_value=60e3,
                               unit=u"ms")
+        self.create_attribute(ATTR_CLOSE_LOOP_UNIT, display=False)
+        self.create_attribute(ATTR_CLOSE_LOOP_FACTOR, default_value=[1, 1], default_type=TYPE_LIST, display=False)
+        self.create_attribute(ATTR_MOVE_METHOD, default_value=CLOSE_LOOP, default_type=[OPEN_LOOP, CLOSE_LOOP],
+                              set_function=self.set_move_method)
 
     def handle_configuration(self):
         if self.configured:
@@ -252,6 +257,8 @@ class DeviceSmaractMcs1Channel(DeviceChannel):
         def _finish_configuration(*_, **__):
             self._config_commands()
             self._config_attributes()
+            self.set_value(ATTR_SENSOR_MODE, self.device.get_value(ATTR_SENSOR_MODE))
+            self.set_value(ATTR_SENSOR_PRESENT, self.device.get_value(ATTR_SENSOR_PRESENT))
             self.get_position_limit_range()
             self.start_polling()
             self.configured = True
@@ -386,6 +393,47 @@ class DeviceSmaractMcs1Channel(DeviceChannel):
                                 callback,
                                 with_token=True)
 
+    def _move_steps(self, value, callback=None):
+        current_position = self.get_value(ATTR_POSITION)
+        if current_position is None:
+            current_position = 0
+        relative_steps = value - current_position
+        n_factor, p_factor = self.get_value(ATTR_CLOSE_LOOP_FACTOR)
+        relative_steps = relative_steps / (n_factor if relative_steps < 0 else p_factor)
+        self.logger.info(f"Move by {relative_steps} steps")
+        if not self.get_value(ATTR_SENSOR_PRESENT) or self.get_value(ATTR_SENSOR_MODE) == "Disabled":
+            self.set_raw_value(ATTR_POSITION, value)
+        return self.move_step(relative_steps, callback=callback)
+
+    def enable_close_loop(self):
+        if self.position_attribute_copy != None:
+            self.set_attribute((ATTR_POSITION, UNIT), self.position_attribute_copy[UNIT])
+            self.set_attribute((ATTR_POSITION, OFFSET), self.position_attribute_copy[OFFSET])
+            self.set_attribute((ATTR_POSITION, FACTOR), self.position_attribute_copy[FACTOR])
+            self[ATTR_POSITION].set_function = self._move_absolute
+            self.position_attribute_copy = None
+
+            # self.poll_command(":CHAN{}:POS:CURR?\r\n".format(self.channel), 250)
+            # self.poll_command(":CHAN{}:POS:TARG?\r\n".format(self.channel), 250)
+
+    def enable_open_loop(self):
+        self.position_attribute_copy = self[ATTR_POSITION].attribute_copy()
+        default_unit = self.get_value(ATTR_CLOSE_LOOP_UNIT)
+
+        self.set_attribute((ATTR_POSITION, UNIT),
+                           default_unit if default_unit != None else self.position_attribute_copy[UNIT])
+        self.set_attribute((ATTR_POSITION, OFFSET), 0)
+        self.set_attribute((ATTR_POSITION, FACTOR), 1)
+        if self.get_value(ATTR_POSITION) is None:
+            self.set_raw_value(ATTR_POSITION, 0)
+        self[ATTR_POSITION].set_function = self._move_steps
+
+    def set_move_method(self, value):
+        if value == OPEN_LOOP:
+            self.enable_open_loop()
+        elif value == CLOSE_LOOP:
+            self.enable_close_loop()
+
     @expose_method()
     def stop(self, callback=None):
         self.logger.info(u"Stop movement")
