@@ -73,6 +73,7 @@ class DeviceSmaractMcs2(DeviceSocket):
     sensor_modes = frozenbidict({"Disabled": 0, "Enabled": 1, "Power safe": 2})
     move_modes = frozenbidict({"Absolute": 0, "Relative": 1, "Scan absolute": 2, "Scan relative": 3, "Step": 4})
     safe_direction_modes = frozenbidict({"Forward": 0, "Backward": 1})
+    emergency_stop_modes = frozenbidict({"Normal": 0, "Restricted": 1, "Auto release": 2})
 
     def __init__(self, host, port=55551, device_id=None, config=None):
         super(DeviceSmaractMcs2, self).__init__(host, port, device_id, config)
@@ -87,6 +88,8 @@ class DeviceSmaractMcs2(DeviceSocket):
         self.create_attribute(ATTR_MOVEMENT_LOCKED, readonly=True, default_type=np.bool)
         self.create_attribute(ATTR_INTERNAL_COMM_FAILURE, readonly=True, default_type=np.bool)
         self.create_attribute(ATTR_STREAMING, readonly=True, default_type=np.bool)
+        self.create_attribute(ATTR_EMERGENCY_STOP, default_type=self.emergency_stop_modes.keys(),
+                              set_function=self.set_emergency_stop_mode)
 
     def command(self, command, callback=None, with_token=False, returning=True):
         if not self.valid_command_format(command):
@@ -103,6 +106,13 @@ class DeviceSmaractMcs2(DeviceSocket):
         command += ":SYST:ERR:NEXT?\r\n"
         super(DeviceSmaractMcs2, self).poll_command(command, interval)
 
+    def remove_poll_command(self, command, interval):
+        if not self.valid_command_format(command):
+            raise CommandFormatException(u"Command '{}' form is invalid".format(command))
+
+        command += ":SYST:ERR:NEXT?\r\n"
+        super(DeviceSmaractMcs2, self).remove_poll_command(command, interval)
+
     def handle_configuration(self):
 
         start_at = time.time()
@@ -121,7 +131,8 @@ class DeviceSmaractMcs2(DeviceSocket):
         self.command(u":DEV:SNUM?\r\n")
         # Get number of modules
         self.command(u":DEV:NOBM?\r\n", callback=_finish_configuration)
-
+        # Get emergency stop mode
+        self.command(u":DEV:EST:MODE?\r\n")
         self.start_polling()
 
     def handle_configuration_event(self):
@@ -191,6 +202,9 @@ class DeviceSmaractMcs2(DeviceSocket):
             self.set_raw_value(ATTR_MOVEMENT_LOCKED, bool(output % 0x0002))
             self.set_raw_value(ATTR_INTERNAL_COMM_FAILURE, bool(output % 0x0100))
             self.set_raw_value(ATTR_STREAMING, bool(output % 0x1000))
+        elif command == ":DEV:EST:MODE?":
+            output = int(output)
+            self.set_raw_value(ATTR_EMERGENCY_STOP, self.emergency_stop_modes.inv[output])
         elif command == ":SYST:ERR:COUN?":
             if int(output) > 0:
                 self.get_next_error()
@@ -275,6 +289,13 @@ class DeviceSmaractMcs2(DeviceSocket):
     def get_next_error(self):
         self.command(u":SYST:ERR:NEXT?\r\n")
 
+    def set_emergency_stop_mode(self, value, callback=None):
+        self.logger.info(u"Set emergency stop mode to {}".format(value))
+        value = self.emergency_stop_modes[value]
+        # Set this mode as a default
+        self.command(u":DEV:EST:MODE:DEF {}\r\n".format(value), callback, with_token=True)
+        return self.command(u":DEV:EST:MODE {}\r\n".format(value), callback, with_token=True)
+
 
 class DeviceSmaractMcs2Channel(DeviceChannel):
     sensor_modes = DeviceSmaractMcs2.sensor_modes
@@ -293,6 +314,7 @@ class DeviceSmaractMcs2Channel(DeviceChannel):
         self.create_attribute(ATTR_CLOSED_LOOP_ACTIVE, readonly=True, default_type=np.bool)
         self.create_attribute(ATTR_CALIBRATING, readonly=True, default_type=np.bool)
         self.create_attribute(ATTR_REFERENCING, readonly=True, default_type=np.bool)
+        self.create_attribute(ATTR_HOLDING, readonly=True, default_type=np.bool)
         self.create_attribute(ATTR_MOVE_DELAYED, readonly=True, default_type=np.bool)
         self.create_attribute(ATTR_SENSOR_PRESENT, readonly=True, default_type=np.bool)
         self.create_attribute(ATTR_CALIBRATED, readonly=True, default_type=np.bool)
@@ -306,7 +328,7 @@ class DeviceSmaractMcs2Channel(DeviceChannel):
         self.create_attribute(ATTR_INDEX_MARK, readonly=True, default_type=np.bool)
         self.create_attribute(ATTR_ENCODER_BASE_UNIT, readonly=True)
         self.create_attribute(ATTR_ENCODER_RESOLUTION, readonly=True, default_type=np.int8)
-        self.create_attribute(ATTR_POSITIONER_CODE, readonly=False, default_type=np.int8, min_value=300, max_value=395,
+        self.create_attribute(ATTR_POSITIONER_CODE, readonly=False, default_type=np.int8, min_value=0, max_value=500,
                               set_function=self.set_positioner_code)
         self.create_attribute(ATTR_POSITIONER_TYPE, readonly=True)
         self.create_attribute(ATTR_POSITION_LOWER_LIMIT, default_type=np.int64,
@@ -328,6 +350,10 @@ class DeviceSmaractMcs2Channel(DeviceChannel):
                               set_function=self.set_safe_direction)
         self.create_attribute(ATTR_HOLDING_TIME, default_type=np.int32, min_value=-1, unit="ms",
                               set_function=self.set_holding_time)
+        self.create_attribute(ATTR_CLOSE_LOOP_UNIT, display=False)
+        self.create_attribute(ATTR_CLOSE_LOOP_FACTOR, default_value=[1, 1], default_type=TYPE_LIST, display=False)
+        self.create_attribute(ATTR_MOVE_METHOD, default_value=CLOSE_LOOP, default_type=[OPEN_LOOP, CLOSE_LOOP],
+                              set_function=self.set_move_method)
 
     def handle_configuration(self):
         if self.configured:
@@ -376,6 +402,14 @@ class DeviceSmaractMcs2Channel(DeviceChannel):
             attribute = key[1]
 
             if attribute == ATTR_STATE and self.get_value(ATTR_STATE) != value:
+                if self.get_value(ATTR_SENSOR_PRESENT) != bool(value & 0x0020):
+                    if bool(value & 0x0020):
+                        self.poll_command(":CHAN{}:POS:CURR?\r\n".format(self.channel), 250)
+                        self.poll_command(":CHAN{}:POS:TARG?\r\n".format(self.channel), 250)
+                    else:
+                        self.remove_poll_command(":CHAN{}:POS:CURR?\r\n".format(self.channel), 250)
+                        self.remove_poll_command(":CHAN{}:POS:TARG?\r\n".format(self.channel), 250)
+
                 self.set_raw_value(ATTR_MOVING, bool(value & 0x0001))
                 self.set_raw_value(ATTR_CLOSED_LOOP_ACTIVE, bool(value & 0x0002))
                 self.set_raw_value(ATTR_CALIBRATING, bool(value & 0x0004))
@@ -392,10 +426,22 @@ class DeviceSmaractMcs2Channel(DeviceChannel):
                 self.set_raw_value(ATTR_OVER_TEMPERATURE, bool(value & 0x4000))
                 self.set_raw_value(ATTR_INDEX_MARK, bool(value & 0x8000))
 
+                holding = bool(value & 0x0004) and bool(value & 0x0008) and bool(value & 0x0008) and bool(
+                    value & 0x0002) \
+                          and not bool(value & 0x0001)
+                self.set_raw_value(ATTR_HOLDING, holding)
+
                 if bool(value & 0x0001):
                     self.set_status(STATUS_BUSY)
                 else:
                     self.set_status(STATUS_IDLE)
+            elif attribute == ATTR_SENSOR_MODE:
+                if value == "Disabled":
+                    self.remove_poll_command(":CHAN{}:POS:CURR?\r\n".format(self.channel), 250)
+                    self.remove_poll_command(":CHAN{}:POS:TARG?\r\n".format(self.channel), 250)
+                else:
+                    self.poll_command(":CHAN{}:POS:CURR?\r\n".format(self.channel), 250)
+                    self.poll_command(":CHAN{}:POS:TARG?\r\n".format(self.channel), 250)
             elif attribute == ATTR_ENCODER_BASE_UNIT:
                 if value == 2:
                     value = u"m"
@@ -405,20 +451,53 @@ class DeviceSmaractMcs2Channel(DeviceChannel):
                 unit = self.get_value(ATTR_ENCODER_BASE_UNIT)
                 prefix = {-12: u"p", -9: u"n", -6: u"u", -3: u"m"}
                 base_position_unit = prefix.get(value, "") + unit
-                self.set_attribute((ATTR_POSITION, UNIT), base_position_unit)
+
                 self.set_attribute((ATTR_TARGET_POSITION, UNIT), base_position_unit)
                 self.set_attribute((ATTR_POSITION_UPPER_LIMIT, UNIT), base_position_unit)
                 self.set_attribute((ATTR_POSITION_LOWER_LIMIT, UNIT), base_position_unit)
                 self.set_attribute((ATTR_ACCELERATION, UNIT), u"{}/s".format(base_position_unit))
                 self.set_attribute((ATTR_VELOCITY, UNIT), u"{}/s".format(base_position_unit))
+                self.set_attribute((ATTR_POSITION, UNIT), base_position_unit)
             elif attribute == ATTR_POSITION_LOWER_LIMIT:
-                self.set_attribute([ATTR_POSITION, MIN], value)
+                self.set_attribute((ATTR_POSITION, MIN), value)
             elif attribute == ATTR_POSITION_UPPER_LIMIT:
-                self.set_attribute([ATTR_POSITION, MAX], value)
-
+                self.set_attribute((ATTR_POSITION, MAX), value)
             self.set_value(ATTR_LATENCY, self.device.get_value(ATTR_LATENCY))
             self.set_raw_value(attribute, value)
 
+    def enable_close_loop(self):
+        if self.position_attribute_copy != None:
+            self.set_attribute((ATTR_POSITION, UNIT), self.position_attribute_copy[UNIT])
+            self.set_attribute((ATTR_POSITION, OFFSET), self.position_attribute_copy[OFFSET])
+            self.set_attribute((ATTR_POSITION, FACTOR), self.position_attribute_copy[FACTOR])
+            self.set_value(ATTR_MOVE_MODE, "Absolute")
+            self[ATTR_POSITION].set_function = self._move_absolute
+            self.position_attribute_copy = None
+
+            # self.poll_command(":CHAN{}:POS:CURR?\r\n".format(self.channel), 250)
+            # self.poll_command(":CHAN{}:POS:TARG?\r\n".format(self.channel), 250)
+
+    def enable_open_loop(self):
+        self.position_attribute_copy = self[ATTR_POSITION].attribute_copy()
+        default_unit = self.get_value(ATTR_CLOSE_LOOP_UNIT)
+
+        self.set_attribute((ATTR_POSITION, UNIT),
+                           default_unit if default_unit != None else self.position_attribute_copy[UNIT])
+        self.set_attribute((ATTR_POSITION, OFFSET), 0)
+        self.set_attribute((ATTR_POSITION, FACTOR), 1)
+
+        self[ATTR_POSITION].set_function = self._move_steps
+        self.set_value(ATTR_MOVE_MODE, "Step")
+
+        # self.remove_poll_command(":CHAN{}:POS:CURR?\r\n".format(self.channel), 250)
+        # self.remove_poll_command(":CHAN{}:POS:TARG?\r\n".format(self.channel), 250)
+
+    def set_move_method(self, value):
+        if value == OPEN_LOOP:
+            self.enable_open_loop()
+        elif value == CLOSE_LOOP:
+            self.enable_close_loop()
+
     @expose_method()
     def stop(self, callback=None):
         self.logger.info(u"Stop movement")
@@ -438,12 +517,10 @@ class DeviceSmaractMcs2Channel(DeviceChannel):
     def move_absolute(self, position):
         try:
             move_mode = self[ATTR_MOVE_MODE][VALUE]
-            position = device_units(self, ATTR_POSITION, position)
+            position = device_units(self, ATTR_POSITION, position).m
             self.logger.info(u"Move to absolute position {}".format(position))
             if move_mode == "Relative":
-                position = position.m - self[ATTR_POSITION][VALUE]
-            else:
-                position = position.m
+                position = position - self[ATTR_POSITION][VALUE]
             return self.set_value(ATTR_TARGET_POSITION, position)
         except (UndefinedUnitError, ValueError) as e:
             self.logger.exception(e)
@@ -457,8 +534,6 @@ class DeviceSmaractMcs2Channel(DeviceChannel):
             self.logger.info(u"Move by relative position {}".format(position))
             if move_mode == "Absolute":
                 position = self[ATTR_POSITION][VALUE] + position
-            else:
-                position = position.m
             return self.set_value(ATTR_TARGET_POSITION, position)
         except (UndefinedUnitError, ValueError) as e:
             self.logger.exception(e)
@@ -466,9 +541,30 @@ class DeviceSmaractMcs2Channel(DeviceChannel):
 
     def _move_absolute(self, value, callback=None):
         self.logger.info(u"Move to position {}".format(value))
+        if self.get_value(ATTR_MOVE_MODE) == "Step":
+            self.set_value(ATTR_MOVE_MODE, "Absolute")
         token = self.command(u":MOVE{} {}\r\n".format(self.channel, value), callback, with_token=True)
         return token
 
+    def _move_steps(self, value, callback=None):
+        if self.get_value(ATTR_MOVE_MODE) != "Step":
+            self.set_value(ATTR_MOVE_MODE, "Step")
+        current_position = self.get_value(ATTR_POSITION)
+        relative_steps = value - current_position
+        n_factor, p_factor = self.get_value(ATTR_CLOSE_LOOP_FACTOR)
+        relative_steps = relative_steps / (n_factor if relative_steps < 0 else p_factor)
+        self.logger.info(f"Move by {relative_steps} steps")
+        if not self.get_value(ATTR_SENSOR_PRESENT) or self.get_value(ATTR_SENSOR_MODE) == "Disabled":
+            self.set_raw_value(ATTR_POSITION, value)
+        token = self.command(u":MOVE{} {}\r\n".format(self.channel, relative_steps), callback, with_token=True)
+        return token
+
+    @expose_method({"position": "New position"})
+    def set_position(self, position=0):
+        self.logger.info(f"Reset position to {position}")
+        position = int(position)
+        self.command(f":CHAN{self.channel}:POS {position}\r\n")
+
     def get_encoder_unit(self):
         self.command(u":CHAN{}:TUN:BASE:UNIT?\r\n".format(self.channel))
         self.command(u":CHAN{}:TUN:BASE:RES?\r\n".format(self.channel))
@@ -530,6 +626,7 @@ class DeviceSmaractMcs2Channel(DeviceChannel):
 
     def set_step_frequency(self, value, callback=None):
         self.logger.info(u"Set step frequency to {} Hz".format(value))
+        self.command(u":CHAN{}:MCLF:CURR {}\r\n".format(self.channel, int(value)), callback, with_token=True)
         return self.command(u":CHAN{}:STEP:FREQ {}\r\n".format(self.channel, int(value)), callback, with_token=True)
 
     def set_step_amplitude(self, value, callback=None):
@@ -547,10 +644,17 @@ class DeviceSmaractMcs2Channel(DeviceChannel):
     def set_sensor_mode(self, value, callback=None):
         self.logger.info(u"Set sensor mode to {}".format(value))
         value = self.sensor_modes[value]
-        return self.command(u":CHAN{}:SENS:MODE {}\r\n".format(self.channel, value), callback, with_token=True)
+        out = self.command(u":CHAN{}:SENS:MODE {}\r\n".format(self.channel, value), callback, with_token=True)
+        self.get_sensor_mode()
+        return out
 
     def set_move_mode(self, value, callback=None):
         self.logger.info(u"Set move mode to {}".format(value))
+        if value == "Step":
+            self.set_value(ATTR_MOVE_METHOD, OPEN_LOOP)
+        else:
+            self.set_value(ATTR_MOVE_METHOD, CLOSE_LOOP)
+
         value = self.move_modes[value]
         return self.command(u":CHAN{}:MMOD {}\r\n".format(self.channel, value), callback, with_token=True)
 
@@ -559,6 +663,34 @@ class DeviceSmaractMcs2Channel(DeviceChannel):
         value = self.safe_direction_modes[value]
         return self.command(u":CHAN{}:SDIR {}\r\n".format(self.channel, value), callback, with_token=True)
 
+    @expose_method()
+    def enable_holding(self, callback=None):
+        """
+        #
+        :param callback:
+        :return:
+        """
+        # enable holding
+        self.set_value(ATTR_HOLDING_TIME, -1)
+        # move by 0 to end holding
+        self.set_value(ATTR_MOVE_MODE, "Relative")
+        self.move_relative(device_units(self, ATTR_POSITION, 0))
+        self.set_value(ATTR_MOVE_MODE, "Absolute")
+
+    @expose_method()
+    def disable_holding(self, callback=None):
+        """
+        #
+        :param callback:
+        :return:
+        """
+        # disable holding
+        self.set_value(ATTR_HOLDING_TIME, 0)
+        # move by 0 to end holding
+        self.set_value(ATTR_MOVE_MODE, "Relative")
+        self.move_relative(device_units(self, ATTR_POSITION, 0))
+        self.set_value(ATTR_MOVE_MODE, "Absolute")
+
 
 class DeviceMCS2FlyScanner(DeviceScanner):
 
