@@ -1,6 +1,6 @@
 import asynchat
 import socket
-from threading import RLock, Lock
+from threading import RLock
 
 from kamzik3.constants import *
 from kamzik3.devices.device import Device
@@ -73,7 +73,6 @@ class DeviceSocket(asynchat.async_chat, Device):
 
     def close_connection(self):
         self.stop_polling()
-        # return asynchat.async_chat.close()
         return self.close()
 
     def close(self):
