@@ -143,9 +143,10 @@ class DeviceVirtualDirectory(DeviceFileServer):
             return True, file_path
         else:
             cached, cache_path = self.is_cached(drive, input_dir, file_name)
-            if cached:
-                return True, cache_path
-            return False, None
+            if os.path.exists(cache_path):
+                return cached, cache_path
+            else:
+                return cached, None
 
     def is_cached(self, drive, input_dir, file_name):
         """
