@@ -25,5 +25,5 @@ class DeviceScanner(Device):
         raise NotImplementedError
 
     @expose_method()
-    def get_scanner_macro(self, scanner_input, scanner_attributes):
+    def get_scanner_macro(self, scanner_input, scanner_attributes, parent_macro=None):
         raise NotImplementedError
