@@ -21,7 +21,7 @@ class GeneralFlyScanner(DeviceScanner):
         return [ATTR_DWELL_TIME]
 
     @expose_method()
-    def get_scanner_macro(self, scanner_input, scanner_attributes):
+    def get_scanner_macro(self, scanner_input, scanner_attributes, parent_macro=None):
         device_id = scanner_input.step_attributes['device_id']
         device = session.get_device(device_id)
         macro = Macro(device_id="{}_{}".format(self.device_id, scanner_input.common_id))
