@@ -11,13 +11,16 @@ from kamzik3.snippets.snippetsYaml import YamlSerializable
 
 class DeviceListWidget(Ui_Form, QWidget, YamlSerializable):
 
-    def __init__(self, columns, title=None, config=None, parent=None):
+    def __init__(self, columns, title=None, default_widgets=None, config=None, parent=None):
         super(DeviceListWidget, self).__init__(parent)
         self.config = config
         self.title = title
         self.widgets = []
         self.columns = columns
         self.setupUi(self)
+        if default_widgets is not None:
+            for widget in default_widgets:
+                self.add_widget(widget)
 
     def yaml_mapping(self):
         mapping = super().yaml_mapping()
@@ -63,7 +66,7 @@ class DeviceListWidget(Ui_Form, QWidget, YamlSerializable):
 class DeviceListWidgetSession(DeviceListWidget):
     sigDeviceCountChanged = pyqtSignal("PyQt_PyObject", "PyQt_PyObject")
 
-    def __init__(self, columns, title=None, parent=None):
+    def __init__(self, columns, title=None, default_widgets=None, parent=None):
         super(DeviceListWidgetSession, self).__init__(columns, title, parent)
         self.sigDeviceCountChanged.connect(self.device_count_changed)
         kamzik3.session.attach_attribute_callback(ATTR_DEVICES_COUNT, self.sigDeviceCountChanged.emit)
