@@ -1,4 +1,4 @@
-from PyQt5.QtCore import pyqtSlot
+from PyQt5.QtCore import pyqtSlot, Qt
 
 from kamzik3 import units
 from kamzik3.constants import *
@@ -26,6 +26,7 @@ class DeviceNumAttributeWidget(Ui_Form, DeviceWidget, StackableWidget):
 
         self.set_status_label(self.device.get_value(ATTR_STATUS))
         self.label_device_name.setText(self.device_id)
+        self.button_stop.setFocusPolicy(Qt.NoFocus)
 
     @pyqtSlot()
     def set_status_label(self, status):
