@@ -132,7 +132,7 @@ class SavedAttributesWidget(Ui_Form, DeviceWidget):
 
     @pyqtSlot("PyQt_PyObject")
     def slot_set_status(self, value):
-        if value in READY_DEVICE_STATUSES:
+        if value in READY_DEVICE_STATUSES and not self.configured:
             self.configured = False
             self.slot_handle_configuration()
             self.device.attach_attribute_callback(ATTR_LAST_UPDATE, self.content_update, key_filter=VALUE)
@@ -370,6 +370,9 @@ class SavedAttributesWidget(Ui_Form, DeviceWidget):
         @param top_group: name of top group
         @type top_group: string
         """
+        if top_group == "":
+            return
+
         self.input_page.blockSignals(True)
         self.input_page.setValue(1)
         self.input_page.blockSignals(False)
