@@ -459,21 +459,14 @@ class ScanAttributeWidget(Ui_Form, MacroWidget):
                 "positive_tolerance": self.attribute.positive_tolerance() if self.user_mode == MODE_SIMPLE else (
                 values["positive_tolerance"] if self.attribute.numerical else None),
                 "retry_count": self.config.get("retry_count", 0) if self.user_mode == MODE_SIMPLE else int(values["retry_count"].m),
-                "trigger_log": self.config.get("trigger_log", True) if self.user_mode == MODE_SIMPLE else self.checkbox_log.isChecked(),
+                "trigger_log": self.checkbox_log.isChecked() if self.user_mode == MODE_ALL else self.config.get("trigger_log", True),
                 "on_warning": self.config.get("on_warning", "Abort macro") if self.user_mode == MODE_SIMPLE else values["on_warning"],
                 "timeout": units.Quantity(self.config.get("timeout", "0s") if self.user_mode == MODE_SIMPLE else values["timeout"]).to("s")
             }
             if values["total_steps"] <= 0:
                 raise ValueError("Total steps for {} has to be greater than zero.".format(self.title))
 
-            scanner = self.config.get("scanner", "None")
-            if scanner == "None":
-                scanner = None
-            scanner_attributes = {}
-            if scanner is not None:
-                scanner_device = kamzik3.session.get_device(scanner)
-                for attribute in scanner_device.get_scanner_attributes():
-                    scanner_attributes[attribute] = self.macro_inputs[attribute].get_attribute_value()
+            scanner, scanner_attributes = self._get_scanner()
 
             as_step = self.checkbox_as_step.isChecked() if self.user_mode == MODE_ALL else self.config.get("as_step", False)
             bidirectional = self.checkbox_bidirectional.isChecked() if self.user_mode == MODE_ALL else self.config.get("bidirectional", False)
