@@ -26,9 +26,10 @@ class Common(YamlSerializable):
     retry_timeout = 100  # 100 ms
     common_id = None
     logger = None
+    resolved = False
 
     def __init__(self, common_id, repeat_count=0, wait_after=units.Quantity(0, "s"), retry_count=0, trigger_log=True,
-                 on_warning=ABORT, timeout=units.Quantity(0, "s")):
+                 on_warning=ABORT, timeout=units.Quantity(0, "s"), scanner=None, scanner_attributes=None):
         assert repeat_count >= 0, u"Repeat parameter should be >= 0"
         assert retry_count >= 0, u"Retry parameter should be >= 0"
         assert wait_after >= 0, u"Wait_after parameter should be >= 0"
@@ -39,6 +40,8 @@ class Common(YamlSerializable):
         self.wait_after = wait_after.to("s")
         self.trigger_log = trigger_log
         self.timeout = timeout.to("s")
+        self.scanner = scanner
+        self.scanner_attributes = scanner_attributes
 
         self.timer = None
         if self.wait_after > 0:
@@ -165,6 +168,8 @@ class Common(YamlSerializable):
     def get_output(self):
         return None
 
+    def on_macro_done(self):
+        return None
 
 class StepGenerator(Common):
     as_step = False
