@@ -109,7 +109,7 @@ class Macro(Common, Device):
                 scanner = kamzik3.session.get_device(link.scanner)
                 new_link = scanner.get_scanner_macro(scanner_input=link, scanner_attributes=link.scanner_attributes,
                                                      parent_macro=self)
-                if isinstance(link, Scan):
+                if isinstance(link, Scan) and hasattr(new_link, "chain"):
                     new_chain += new_link.chain
                 else:
                     new_chain.append(new_link)
