@@ -13,11 +13,12 @@ from kamzik3.macro.common import Common
 class Step(Common):
 
     def __init__(self, common_id, repeat_count=0, wait_after=units.Quantity(0, "s"), retry_count=0, trigger_log=True,
-                 on_warning=ABORT, timeout=units.Quantity(0, "s")):
+                 on_warning=ABORT, timeout=units.Quantity(0, "s"), scanner=None, scanner_attributes=None):
         self.steps_count = 1
         if not hasattr(self, "logger"):
             self.logger = logging.getLogger("Macro.Step".format(common_id))
-        super(Step, self).__init__(common_id, repeat_count, wait_after, retry_count, trigger_log, on_warning, timeout)
+        super(Step, self).__init__(common_id, repeat_count, wait_after, retry_count, trigger_log, on_warning, timeout,
+                                   scanner=scanner, scanner_attributes=scanner_attributes)
 
     def start(self):
         super(Step, self).start()
@@ -83,12 +84,13 @@ class Step(Common):
 class StepSetpoint(Step):
 
     def __init__(self, common_id, setpoint, repeat_count=0, wait_after=units.Quantity(0, "s"), retry_count=0,
-                 trigger_log=True, on_warning=ABORT, timeout=units.Quantity(0, "s")):
+                 trigger_log=True, on_warning=ABORT, timeout=units.Quantity(0, "s"), scanner=None,
+                 scanner_attributes=None):
         self.setpoint = setpoint
         if not hasattr(self, "logger"):
             self.logger = logging.getLogger("Macro.Step".format(common_id))
         super(StepSetpoint, self).__init__(common_id, repeat_count, wait_after, retry_count, trigger_log, on_warning,
-                                           timeout)
+                                           timeout, scanner=scanner, scanner_attributes=scanner_attributes)
 
     def body(self):
         raise NotImplementedError()
@@ -112,7 +114,8 @@ class StepDeviceAttribute(StepSetpoint):
     device_attribute = None
 
     def __init__(self, common_id, device_id, attribute, setpoint, repeat_count=0, wait_after=units.Quantity(0, "s"),
-                 retry_count=0, trigger_log=True, on_warning=ABORT, timeout=units.Quantity(0, "s")):
+                 retry_count=0, trigger_log=True, on_warning=ABORT, timeout=units.Quantity(0, "s"), scanner=None,
+                 scanner_attributes=None):
         self.device_id = device_id
         self.attribute = Attribute.list_attribute(attribute)
         self.device_status_idle_flag = False
@@ -122,7 +125,8 @@ class StepDeviceAttribute(StepSetpoint):
         if not hasattr(self, "logger"):
             self.logger = logging.getLogger("Macro.Step.{}.{}.{}".format(common_id, device_id, attribute))
         super(StepDeviceAttribute, self).__init__(common_id, setpoint, repeat_count, wait_after, retry_count,
-                                                  trigger_log, on_warning, timeout)
+                                                  trigger_log, on_warning, timeout, scanner=scanner,
+                                                  scanner_attributes=scanner_attributes)
 
     def body(self):
         self.check_timeout = 0
@@ -224,7 +228,7 @@ class StepSetpointNumerical(Step):
 
     def __init__(self, common_id, setpoint, negative_tolerance=None, positive_tolerance=None, repeat_count=0,
                  wait_after=units.Quantity(0, "s"), retry_count=0, trigger_log=True, on_warning=ABORT,
-                 timeout=units.Quantity(0, "s")):
+                 timeout=units.Quantity(0, "s"), scanner=None, scanner_attributes=None):
         assert isinstance(setpoint, units.Quantity)
 
         self.setpoint = setpoint
@@ -237,7 +241,8 @@ class StepSetpointNumerical(Step):
         if not hasattr(self, "logger"):
             self.logger = logging.getLogger("Macro.Step".format(common_id))
         super(StepSetpointNumerical, self).__init__(common_id, repeat_count, wait_after, retry_count, trigger_log,
-                                                    on_warning, timeout)
+                                                    on_warning, timeout, scanner=scanner,
+                                                    scanner_attributes=scanner_attributes)
 
     def body(self):
         raise NotImplementedError()
@@ -270,7 +275,7 @@ class StepDeviceAttributeNumerical(StepSetpointNumerical):
 
     def __init__(self, common_id, device_id, attribute, setpoint, negative_tolerance=None, positive_tolerance=None,
                  repeat_count=0, wait_after=units.Quantity(0, "s"), retry_count=0, trigger_log=True, on_warning=ABORT,
-                 timeout=units.Quantity(0, "s")):
+                 timeout=units.Quantity(0, "s"), scanner=None, scanner_attributes=None):
         self.device_id = device_id
         self.attribute = Attribute.list_attribute(attribute)
         self.device_status_idle_flag = False
@@ -281,7 +286,8 @@ class StepDeviceAttributeNumerical(StepSetpointNumerical):
             self.logger = logging.getLogger("Macro.Step.{}.{}.{}".format(common_id, device_id, attribute))
         super(StepDeviceAttributeNumerical, self).__init__(common_id, setpoint, negative_tolerance, positive_tolerance,
                                                            repeat_count, wait_after, retry_count, trigger_log,
-                                                           on_warning, timeout)
+                                                           on_warning, timeout, scanner=scanner,
+                                                           scanner_attributes=scanner_attributes)
 
     def body(self):
         self.check_timeout = 0
@@ -396,7 +402,8 @@ class StepDeviceMethod(Step):
 
     def __init__(self, common_id, device_id, method, method_parameters, repeat_count=0,
                  wait_after=units.Quantity(0, "s"), retry_count=0, busy_timeout=units.Quantity(1, "s"),
-                 trigger_log=True, on_warning=ABORT, timeout=units.Quantity(0, "s")):
+                 trigger_log=True, on_warning=ABORT, timeout=units.Quantity(0, "s"), scanner=None,
+                 scanner_attributes=None):
         self.device_id = device_id
         self.method = method
         self.method_parameters = method_parameters
@@ -406,7 +413,8 @@ class StepDeviceMethod(Step):
         self.busy_timeout = busy_timeout
         if not hasattr(self, "logger"):
             self.logger = logging.getLogger("Macro.Step.{}.{}.{}".format(common_id, device_id, method))
-        Step.__init__(self, common_id, repeat_count, wait_after, retry_count, trigger_log, on_warning, timeout)
+        Step.__init__(self, common_id, repeat_count, wait_after, retry_count, trigger_log, on_warning, timeout,
+                      scanner=scanner, scanner_attributes=scanner_attributes)
 
     def body(self):
         self.check_timeout = 0
@@ -492,13 +500,15 @@ class StepDeviceMethod(Step):
 class StepParallel(Step):
 
     def __init__(self, common_id, step_list, repeat_count=0, wait_after=units.Quantity(0, "s"), retry_count=0,
-                 trigger_log=True, on_warning=ABORT, timeout=units.Quantity(0, "s")):
+                 trigger_log=True, on_warning=ABORT, timeout=units.Quantity(0, "s"), scanner=None,
+                 scanner_attributes=None):
         assert isinstance(step_list, list)
         self.step_list = step_list
         for step in self.step_list:
             if not isinstance(step, Step):
                 raise TypeError("Step {} is not type of Step".format(step))
-        Step.__init__(self, common_id, repeat_count, wait_after, retry_count, trigger_log, on_warning, timeout)
+        Step.__init__(self, common_id, repeat_count, wait_after, retry_count, trigger_log, on_warning, timeout,
+                      scanner=scanner, scanner_attributes=scanner_attributes)
 
     def body(self):
         step_threads = []
