@@ -28,8 +28,10 @@ def set_rotating_file_handler(logger, log_output_dir, formatter=base_log_formatt
     return handler
 
 
-def set_rocket_chat_handler(logger, level=logging.ERROR, formatter=base_log_formatter):
-    handler = RocketChatHandler(user='kam3logger', password='WmaeX5ukaZ5p6FRV', level=level)
+def set_rocket_chat_handler(logger, main_room, discussion, user_credentials=None, auth_token=None,
+                            level=logging.ERROR, formatter=base_log_formatter):
+    handler = RocketChatHandler(user_credentials=user_credentials, auth_token=auth_token, main_room_name=main_room,
+                                discussion_name=discussion, level=level)
     handler.setFormatter(formatter)
     logger.addHandler(handler)
     return handler
@@ -43,6 +45,12 @@ def get_console_handler(log_level=logging.DEBUG, formatter=base_log_formatter):
 
 
 def set_sys_exception_handler(logger):
+    """
+    Pass all Uncaught exception into logger handler
+    :param logger:
+    :return:
+    """
+
     def handle_exception(exc_type, exc_value, exc_traceback):
         logger.error("Uncaught exception", exc_info=(exc_type, exc_value, exc_traceback))
         sys.__excepthook__(exc_type, exc_value, exc_traceback)
@@ -51,30 +59,77 @@ def set_sys_exception_handler(logger):
 
 
 def print_exception():
+    """
+    Print formatted exception in stdout
+    :return: None
+    """
     print(traceback.format_exc())
 
 
 class RocketChatHandler(Handler):
 
-    def __init__(self, user, password, channel="kamzik3", server_url="https://cfel-rocketchat.desy.de",
-                 level=logging.NOTSET):
+    def __init__(self, user_credentials=None, auth_token=None, main_room_name="kamzik3", discussion_name="kamzik3",
+                 server_url="https://cfel-rocketchat.desy.de", level=logging.NOTSET):
+
+        # Import rocketchat library
         from rocketchat_API.rocketchat import RocketChat
         from requests import sessions
-        self.channel = channel
+
+        self.discussion_name = discussion_name
+        self.main_room_name = main_room_name
+
+        # Check if we have any user credentials or access token
+        if (user_credentials is None or (user_credentials[0] == user_credentials[1] is None)) and auth_token is None:
+            raise Exception(f"No user authentication method was provided!")
+
+        # Check if session was already open
         if not hasattr(kamzik3, "rc_session"):
             kamzik3.rc_session = sessions.Session()
-            kamzik3.rc = RocketChat(user, password, server_url=server_url, session=kamzik3.rc_session)
+            kamzik3.rc = RocketChat(*user_credentials, auth_token=auth_token, server_url=server_url,
+                                    session=kamzik3.rc_session)
+
+        # Check if Discussion already exists
+        all_rooms = kamzik3.rc.rooms_get().json()
+        self.main_room_id = None
+        self.discussion_room_id = None
+        self.available_discussions = []
+        # Loop over all groups, that user has access to
+        for room in all_rooms["update"]:
+            # We found main room
+            if room.get("name") == self.main_room_name:
+                self.main_room_id = room["_id"]
+                # Get all discussions within room
+                room_discussions = kamzik3.rc.call_api_get("chat.getDiscussions", roomId=self.main_room_id).json()
+                for discussion in room_discussions["messages"]:
+                    # Discussion was found
+                    if discussion["msg"] == self.discussion_name:
+                        self.discussion_room_id = discussion["drid"]
+                        break
+                break
+
+        # Throw Exception if main room does not exists
+        if self.main_room_id is None:
+            raise Exception(f"RocketChat {self.main_room_name} room does not exists or user has no access right!")
+        # Throw Exception if discussion does not exists
+        if self.discussion_room_id is None:
+            raise Exception(
+                f"RocketChat {self.discussion_name} discussion does not exists or user has no access right!")
+
         Handler.__init__(self, level)
 
     def emit(self, record):
+        """
+        Write logged message in RocketChat Discussion
+        :param record: str
+        :return: None
+        """
         if record.exc_info is not None:
             msg = "```" + self.format(record) + "```" + record.getMessage()
         elif record.levelno == logging.ERROR:
             msg = "```" + traceback.format_exc() + "```" + record.getMessage()
         else:
             msg = record.getMessage()
-
-        kamzik3.rc.chat_post_message(msg, channel='kamzik3').json()
+        kamzik3.rc.chat_post_message(msg, room_id=self.discussion_room_id).json()
 
 
 class MeasureTiming:
