@@ -141,6 +141,25 @@ class GlSwitchTexture(GlWidget):
         return GlWidget.draw(self, painter)
 
 
+class GlMappedTexture(GlWidget):
+    key = 0
+
+    def __init__(self, parent, widget_id, coords, texture_map):
+        GlWidget.__init__(self, parent, widget_id, coords)
+        self.texture_map = texture_map
+
+    def set_state(self, state):
+        return self.set_key(state)
+
+    def set_key(self, key):
+        self.key = key
+        return self
+
+    def draw(self, painter):
+        self.texture = self.texture_map[self.key]
+        return GlWidget.draw(self, painter)
+
+
 class GlSwitchColor(GlWidget):
     state = 0
 
@@ -163,3 +182,26 @@ class GlSwitchColor(GlWidget):
         else:
             self.background = (coords, self.off_color, shape)
         return GlWidget.draw(self, painter)
+
+
+class GlMappedColor(GlWidget):
+    key = 0
+
+    def __init__(self, parent, widget_id, coords, color_map):
+        GlWidget.__init__(self, parent, widget_id, coords)
+        self.color_map = color_map
+
+    def set_state(self, state):
+        return self.set_key(state)
+
+    def set_key(self, key):
+        self.key = key
+        return self
+
+    def draw(self, painter):
+        if self.background is not None:
+            coords, _, shape = self.background
+        else:
+            coords, shape = self.coords, "rect"
+        self.background = (coords, self.color_map[self.key], shape)
+        return GlWidget.draw(self, painter)
