@@ -69,13 +69,16 @@ def get_decimals_diff(old_unit, new_unit):
 
 
 def device_units(device, attribute, value):
+    value = str(value)
     try:
         value = value.replace(u"%", u"percent")
     except AttributeError:
         pass
-    device_unit = device.get_attribute(attribute)[UNIT]
-    # try:
     converted_value = units.Quantity(value)
+
+    device_unit = device.get_attribute(attribute)[UNIT]
+    # try to convert value:
+
     # No unit for value
     if converted_value.unitless:
         # Return in value in device units
@@ -90,9 +93,6 @@ def device_units(device, attribute, value):
     else:
         # Return converted value
         return converted_value.to(device_unit)
-    # except DimensionalityError:
-    #     # Values are not convertable among each other
-    #     return units.Quantity(value, device_unit)
 
 
 MAX_SECONDS = 3.154e+9
